﻿using System.Net.NetworkInformation;
using System.Net.Sockets;
using System.Text;
using Autofac;
using BPA.Component.ApolloClient;
using BPA.Component.CAP;
using BPA.Component.Extensions;
using BPA.Component.LogClient.Extensions;
using BPA.Component.SDKCommon;
using BPA.Component.WebApiExtensions.Extensions;
using BPA.SaaS.Material.Api.Model;
using BPA.SaaS.Material.Api.Repository;
using BPA.SaaS.Material.Api.Service;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Diagnostics.HealthChecks;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.OpenApi.Models;
using IPAddress = System.Net.IPAddress;

namespace BPA.SaaS.Material.Api.Bootstrap;

/// <summary>
/// 启动配置
/// </summary>
public static class SetupBootstrap
{
    private static IConfiguration _configuration;

    /// <summary>
    /// 添加配置中心
    /// </summary>
    /// <param name="configuration"></param>
    public static IConfiguration SetupConfiguration(this IConfiguration configuration)
    {
        _configuration = configuration;

        // 初始化化 apollo配置
        return configuration.AddApolloConfiguration();
    }

    /// <summary>
    /// DI Inject
    /// </summary>
    public static IServiceCollection AddBPAMiddlewareServices(this IServiceCollection services, string[] args)
    {
        if (services == null) throw new ArgumentNullException(nameof(services));

        services.AddApollo<MaterialApiConfig>();
        services.AddBPALog();
        services.AddRedis();
        services.AddRabbitMQ();
        services.AddHttpContextAccessor();
        services.AddControllers(options => { options.AddDefaultMvcOptions(); });
        services.AddEndpointsApiExplorer();
        services.AddServiceSDK(_configuration);
        services.AddHealthChecks();

        // services.AddCap(x =>
        // {
        //     var config = services.GetPreConfig(args);
        //     x.UseRabbitMQ(mqOptions =>
        //     {
        //         mqOptions.UserName = config.RabbitMqConfig.UserName;
        //         mqOptions.Password = config.RabbitMqConfig.Password;
        //         mqOptions.Port = config.RabbitMqConfig.Port;
        //         mqOptions.HostName = config.RabbitMqConfig.HostName;
        //         mqOptions.VirtualHost = config.RabbitMqConfig.VirtualHost;
        //     });
        //     x.UseMySql(config.BasicMySqlDb.MasterDbConnectionString);
        // });

        services.AddSwaggerGen(options =>
        {
            options.SwaggerDoc("v1", new OpenApiInfo
            {
                Title = AppDomain.CurrentDomain.FriendlyName,
                Description = AppDomain.CurrentDomain.FriendlyName,
                Version = "v1"
            });
            options.CustomSchemaIds(type => type.GetDescribe(true, "BPA.", "BPA.SaaS."));
        });

        return services;
    }

    /// <summary>
    /// DI Register
    /// </summary>
    public static ContainerBuilder SetupConfigureContainer(this ContainerBuilder builder)
    {
        // 注入DB
        builder.RegisterType<MaterialDbSugarClient>().InstancePerLifetimeScope();

        // 服务层
        builder.RegisterAssemblyTypes(typeof(UnitService).Assembly)
            .Where(t => t.Name.EndsWith("Service"))
            .AsImplementedInterfaces();

        // 仓储层
        builder.RegisterAssemblyTypes(typeof(UnitRepository).Assembly)
            .Where(t => t.Name.EndsWith("Repository"))
            .AsImplementedInterfaces();

        return builder;
    }

    /// <summary>
    /// pre run
    /// </summary>
    /// <param name="serviceProvider"></param>
    public static void PreHeatRun(this IServiceProvider serviceProvider)
    {
        serviceProvider.GetService<MaterialApiConfig>();
        serviceProvider.GetService<MaterialDbSugarClient>()?.DbMaintenance.GetTableInfoList();
        serviceProvider.PreHeatRunRedis();
        serviceProvider.PreHeatRunRabbitMQ();
        serviceProvider.SetDebug(true);
        serviceProvider.GetService<ICapPublisher>();
    }

    /// <summary>
    /// APP健康检查
    /// </summary>
    /// <param name="app"></param>
    public static void UseBpaHealthChecks(this IApplicationBuilder app)
    {
        var ips = new List<dynamic>();
        foreach (var network in NetworkInterface.GetAllNetworkInterfaces())
        {
            var ipAddress = network.GetIPProperties()
                .UnicastAddresses
                .FirstOrDefault(p => p.Address.AddressFamily == AddressFamily.InterNetwork && !IPAddress.IsLoopback(p.Address))
                ?.Address.ToString();
            if (ipAddress == null)
                continue;
            ips.Add(new {name = network.Name, ip = ipAddress});
        }

        var ipsJson = ips.ToJson();

        app.UseHealthChecks("/health", new HealthCheckOptions
        {
            ResponseWriter = (httpContext, _) => httpContext.Response.WriteAsync(ipsJson, Encoding.UTF8)
        });
    }

    private static MaterialApiConfig GetPreConfig(this IServiceCollection services, string[] args)
    {
        var tmpBuilder = WebApplication.CreateBuilder(args);
        tmpBuilder.Configuration.SetupConfiguration();
        tmpBuilder.Services.AddApollo<MaterialApiConfig>();
        tmpBuilder.Services.AddBPALog();
        var tmpApp = tmpBuilder.Build();
        var config = tmpApp.Services.GetService<MaterialApiConfig>();
        tmpApp.StopAsync().ConfigureAwait(false).GetAwaiter();
        tmpApp.DisposeAsync().ConfigureAwait(false).GetAwaiter();
        tmpBuilder = null;
        tmpApp = null;
        return config;
    }
}